/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.view.fx;

import static javafx.scene.paint.Color.BISQUE;
import static javafx.scene.paint.Color.LIGHTGREY;
import static javafx.scene.paint.Color.LIGHTSEAGREEN;
import static javafx.scene.paint.Color.WHITE;
import static org.eclipse.swt.widgets.Display.getCurrent;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.computeFullyQualifiedName;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.computeRelativeName;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.graphics.Device;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.annotation.valueprovider.IAnnotationValueProvider;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.model.INamedElement;

import javafx.scene.paint.Color;

/**
 * Utils for the FX implementation of the annotations view.
 * 
 * @author bayha
 */
public class AnnotationsFXUtils {

	/** Background color for table cells, which are not editable and empty. */
	public static final Color READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR = LIGHTGREY;

	/**
	 * SWT background color for table cells, which are not editable and empty.
	 * 
	 * Deprecated - should be removed, when SWT is completely replaced.
	 */
	@Deprecated
	public static final org.eclipse.swt.graphics.Color READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR_SWT =
			convertFXColortoSWT(READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR);

	/** Background color for table cells, which are not editable. */
	public static final Color READ_ONLY_ANNOTATION_BACKGROUND_COLOR = BISQUE;

	/** Background color for table cells of the selected element. */
	protected static final Color SELECTED_ANNOTATION_BACKGROUND_COLOR = LIGHTSEAGREEN;

	/**
	 * Retrieves the background {@link Color} to be used for the given
	 * {@link AnnotationEntry}.
	 * 
	 * @param annotationEntry
	 *            The {@link AnnotationEntry} to get a background color for.
	 * @param annotationType
	 *            The {@link Class} of the {@link IAnnotatedSpecification}.
	 * 
	 * @return The {@link Color} to be used in the background.
	 */
	public static Color getBackgroundColorForEntry(AnnotationEntry annotationEntry,
			Class<? extends IAnnotatedSpecification> annotationType) {
		IAnnotationValueProvider<IAnnotatedSpecification> valueProvider =
				annotationEntry.getAnnotationValueProvider(annotationType);
		IAnnotatedSpecification specification = annotationEntry.getSpecification(annotationType);

		if(valueProvider != null && valueProvider.getAnnotationValue(specification) != null &&
				!valueProvider.canEdit(specification)) {
			return READ_ONLY_ANNOTATION_BACKGROUND_COLOR;
		}

		if(valueProvider == null || !valueProvider.canEdit(specification)) {
			return READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR;
		}

		return WHITE;
	}

	/**
	 * Retrieves the background {@link org.eclipse.swt.graphics.Color} for the given
	 * {@link AnnotationEntry}.
	 * A default {@link org.eclipse.swt.graphics.Color} if there is nothing other than white
	 * specified, yet.
	 * 
	 * Deprecated - should be removed, when SWT is completely replaced.
	 * 
	 * @param annotationEntry
	 *            The {@link AnnotationEntry} to get a background color for.
	 * @param annotationType
	 *            The {@link Class} of the {@link IAnnotatedSpecification}.
	 * @param defaultColor
	 *            The default {@link org.eclipse.swt.graphics.Color} to be used, if it would be
	 *            white otherwise. Might be null if WHITE shall be used.
	 * 
	 * @return The background {@link org.eclipse.swt.graphics.Color} to be used for the given input.
	 */
	@Deprecated
	public static org.eclipse.swt.graphics.Color getSWTBackgroundColorForEntry(
			AnnotationEntry annotationEntry,
			Class<? extends IAnnotatedSpecification> annotationType,
			org.eclipse.swt.graphics.Color defaultColor) {
		Color bgCol = getBackgroundColorForEntry(annotationEntry, annotationType);

		if(bgCol.equals(WHITE)) {
			return defaultColor;
		}

		return convertFXColortoSWT(bgCol);
	}

	/**
	 * Helper to convert JavaFx Color objects to SWT.
	 * 
	 * Deprecated - should be removed, when SWT is completely replaced.
	 * 
	 * @param color
	 *            The FX {@link Color} to be converted.
	 * 
	 * @return The input, converted to a SWT {@link org.eclipse.swt.graphics.Color}.
	 */
	@Deprecated
	private static org.eclipse.swt.graphics.Color convertFXColortoSWT(Color color) {
		Device device = getCurrent();
		int red = (int)color.getRed();
		int green = (int)color.getGreen();
		int blue = (int)color.getBlue();
		return new org.eclipse.swt.graphics.Color(device, red, green, blue);
	}

	/**
	 * Returns the label for a value shown in the annotation view. For values of EReferences, the
	 * hierarchical name of the value relative to the given {@code root} model element is returned.
	 */
	public static String getValueLabel(Object modelELement, EObject root) {
		if(modelELement instanceof INamedElement) {
			if(root instanceof INamedElement) {
				return computeRelativeName((INamedElement)root, (INamedElement)modelELement);
			}
			return computeFullyQualifiedName((INamedElement)modelELement);
		}
		return modelELement.toString();
	}
}
